@@ -127,8 +127,7 @@ DataProvider::DataProvider(
 #ifdef WITH_WSREP
       db_replication_id_solver_(std::make_shared<DbReplicationIdSolver>()),
       gcache_encryption_enabled_cache_(-1),
-      ws_cache_encryption_enabled_cache_(-1) {
-}
+      ws_cache_encryption_enabled_cache_(-1) {}
 #else
       db_replication_id_solver_(std::make_shared<DbReplicationIdSolver>()) {}
 #endif
