@@ -55,7 +55,6 @@ set @@persist_only.log_replica_updates=1;
 set @@persist_only.max_digest_length=1024;
 set @@persist_only.myisam_recover_options='OFF';
 set @@persist_only.ngram_token_size=2;
-set @@persist_only.old=0;
 set @@persist_only.open_files_limit=5000;
 set @@persist_only.performance_schema=1;
 set @@persist_only.performance_schema_accounts_size=-1;
@@ -89,7 +88,7 @@ set @@persist_only.performance_schema_max_socket_classes=10;
 set @@persist_only.performance_schema_max_socket_instances=-1;
 set @@persist_only.performance_schema_max_sql_text_length=1024;
 set @@persist_only.performance_schema_max_stage_classes=250;
-set @@persist_only.performance_schema_max_statement_classes=202;
+set @@persist_only.performance_schema_max_statement_classes=256;
 set @@persist_only.performance_schema_max_statement_stack=10;
 set @@persist_only.performance_schema_max_table_handles=-1;
 set @@persist_only.performance_schema_max_table_instances=-1;
@@ -109,10 +108,10 @@ set @@persist_only.table_open_cache_instances=16;
 set @@persist_only.thread_handling='one-thread-per-connection';
 set @@persist_only.thread_stack=286720;
 set @@persist_only.tls_version='TLSv1.2,TLSv1.3';
-set @@persist_only.report_host=NULL;
+set @@persist_only.report_host='';
 set @@persist_only.report_port=21000;
-set @@persist_only.report_password=NULL;
-set @@persist_only.report_user=NULL;
+set @@persist_only.report_password='';
+set @@persist_only.report_user='';
 
 # Added by Percona
 call mtr.add_suppression('Default storage engine \\(InnoDB\\) is not the same as enforced storage engine \\(MyISAM\\)');
