@@ -107,7 +107,8 @@ namespace galera
         void process_apply_error(TrxHandleSlave&, const wsrep_buf_t&);
 
         wsrep_status_t send(TrxHandleMaster& trx, wsrep_trx_meta_t*);
-        wsrep_status_t replicate(TrxHandleMaster& trx, wsrep_trx_meta_t*);
+        wsrep_status_t replicate(TrxHandleMaster& trx, wsrep_trx_meta_t*,
+                                 const wsrep_seq_cb_t* seq_cb);
         wsrep_status_t abort_trx(TrxHandleMaster& trx, wsrep_seqno_t bf_seqno,
                                  wsrep_seqno_t* victim_seqno);
         wsrep_status_t certify(TrxHandleMaster& trx, wsrep_trx_meta_t*);
@@ -460,8 +461,8 @@ namespace galera
         {
             if (gu_unlikely(purge_seqno != -1))
             {
-                assert(purge_seqno <= last_committed());
-                service_thd_.report_last_committed(purge_seqno);
+                service_thd_.report_last_committed(
+                    std::min(purge_seqno, last_committed()));
             }
         }
         // Helpers for configuration change processing
