@@ -78,7 +78,7 @@ ReplicatorSMM::sst_received(const wsrep_gtid_t& state_id,
                             int                const rcode)
 {
 #ifdef PXC
-    if (rcode != -ECANCELED && rcode != -EPIPE)
+    if (rcode != -ECANCELED && rcode != -EPIPE && rcode != -EAGAIN)
     {
         log_info << "SST received: " << state_id.uuid << ':' << state_id.seqno;
     }
@@ -113,17 +113,28 @@ ReplicatorSMM::sst_received(const wsrep_gtid_t& state_id,
     sst_cond_.signal();
 
 #ifdef PXC
-    // If the donor server crashed while SST is in progress, then SST script
-    // aborts with error 32 (Broken pipe) after the timeout is exceeded.  When
-    // this happens there is nothing that we can to do recover. So, we must
-    // abort.
-    if (rcode == -EPIPE)
-    {
-        log_fatal << "State transfer request failed unrecoverably: "
-                  << -rcode << " (" << strerror(-rcode) << "). Most likely "
-                  << "it is due to inability to communicate with the "
-                  << "cluster primary component. Restart required.";
-        abort();
+    /* SST script may exit with EAGAIN code. This is the hint that data
+     directory was not deleted and it is up to caller to decide what to do.
+     (eg. SST script pre-check failed because of some external binary missing
+     or because sst-info file was not received because of network failure)
+     Here, as the caller, we decide to mark back Galera state as safe to avoid
+     forced SST next time, but try with IST */
+    switch (rcode) {
+        case -EAGAIN:
+            log_fatal << "SST finished with code: " << -rcode << ". "
+                    << "It means that SST failed before wiping out the data "
+                    << "directory. Saving node state to retry with IST instead "
+                    << "of full SST after restart.";
+            st_.mark_safe();
+            [[fallthrough]];
+        case -EPIPE:
+            log_fatal << "State transfer request failed unrecoverably: "
+                    << -rcode << " (" << strerror(-rcode) << "). Most likely "
+                    << "it is due to inability to communicate with the "
+                    << "cluster primary component. Please check the error log "
+                    << "for details. Restart required.";
+            abort();
+            break;
     }
 
     // We need to check the state only after we signalized about completion
@@ -682,7 +693,33 @@ out:
 
     local_monitor_.leave(lo);
 
-    if (join_now || rcode < 0)
+    /*
+     During the State Transfer (ST) we always go though SST script. It's
+     responsibility is to send the info to the Joiner if SST can be bypassed
+     and ST can be handled by IST. Let's consider the situation when Joiner
+     requests SST by starting SST script and IST Receiver and waits for
+     the info what will be transferred (SST+IST or only IST). At the same
+     time, on Donor side there is some delay, and sending this info to Joiner
+     is delayed.
+     Joiner timeouts and exits (aborts), because it can't get the ST. Then
+     Donor goes ahead. It starts SST script which tries to send the info that
+     only IST will be sent and starts IST Sender. But because the Joiner's
+     IST Receiver doesn't listen anymore, the Donor's IST Sender gets
+     -ECONNREFUSED.
+     In such a case, on Donor side if we shift Galera to JOINED directly
+     after the failure without waiting for SST script to finish, we will go
+     through states: DONOR->JOINED->SYNCED.
+     Unfortunately shifting to JOINED is not signalized to application
+     (wsrep_lib), therefore wsrep_lib will see the transition DONOR->SYNCED,
+     which is prohibited on that layer.
+
+     In normal case, wsrep_lib layer is shifted to JOINED state from donor
+     thread context, when SST is finished (SST script serverd SST or informed
+     Joiner that it will serve only IST) in server_state::sst_sent() which
+     will call ReplicatorSMM::sst_sent(), where Galera will be shifted to
+     JOINED state.
+    */
+    if (join_now || (rcode < 0 && rcode != -ECONNREFUSED))
     {
         gcs_.join(gu::GTID(state_uuid_, donor_seq), rcode);
     }
