@@ -152,7 +152,7 @@ void run_wsinfo(const WSInfo* const wsi, size_t const nws, int const version)
                           buf,
                           static_cast<int32_t>(size),
                           GCS_ACT_WRITESET,
-                          0};
+                          {0}};
         galera::TrxHandleSlavePtr ts(galera::TrxHandleSlave::New(false, sp),
                                      galera::TrxHandleSlaveDeleter());
         ck_assert(ts->unserialize<true>(act) == size);
@@ -706,7 +706,7 @@ struct CertFixture
         ck_assert(out.serialize(buf, size) == size);
         ++cur_seqno;
         gcs_action act = { cur_seqno, cur_seqno, buf,
-                           static_cast<int32_t>(size), GCS_ACT_WRITESET, 0 };
+                           static_cast<int32_t>(size), GCS_ACT_WRITESET, {0} };
         galera::TrxHandleSlavePtr ts(galera::TrxHandleSlave::New(false, sp),
                                      galera::TrxHandleSlaveDeleter{});
         ck_assert(ts->unserialize<true>(act) == size);
