@@ -1,4 +1,4 @@
-/* Copyright (C) 2011-2013 Codership Oy <info@codership.com> */
+/* Copyright (C) 2011-2025 Codership Oy <info@codership.com> */
 
 #include "garb_config.hpp"
 #include "garb_logger.hpp"
@@ -180,7 +180,8 @@ Config::Config (int argc, char* argv[])
     strip_quotes(cfg_);
 
     if (options_.length() > 0) options_ += "; ";
-    options_ += "gcs.fc_limit=9999999; gcs.fc_factor=1.0; gcs.fc_single_primary=yes";
+    options_ += "gcs.fc_limit=9999999; gcs.fc_factor=1.0;"
+                "gcs.fc_single_primary=yes; gcs.stateless=yes;";
     if (!workdir_.empty())
     {
         options_ += " base_dir=" + workdir_ + ";";
