@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2018 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2025 Codership Oy <info@codership.com>
  */
 
 /*! @file page file class implementation */
@@ -56,7 +56,13 @@ gcache::Page::drop_fs_cache() const
 #endif
 }
 
-gcache::Page::Page (void* ps, const std::string& name, size_t size, int dbg, bool encrypt, size_t encrypt_cache_page_size, size_t encrypt_cache_size)
+gcache::Page::Page (void* const        ps,
+                    const std::string& name,
+                    size_t             size,
+                    int                dbg,
+                    bool               encrypt,
+                    size_t             encrypt_cache_page_size,
+                    size_t encrypt_cache_size)
     :
 #ifdef PXC
 #ifdef HAVE_PSI_INTERFACE
@@ -69,6 +75,7 @@ gcache::Page::Page (void* ps, const std::string& name, size_t size, int dbg, boo
 #endif /* PXC */
     mmapptr_   (gu::MMapFactory::create(fd_, encrypt, encrypt_cache_page_size, encrypt_cache_size, false, 0)),
     mmap_      (*mmapptr_),
+    seqno_max_(SEQNO_NONE),
     ps_   (ps),
     next_ (static_cast<uint8_t*>(mmap_.get_ptr())),
     space_(mmap_.get_size()),
@@ -77,7 +84,8 @@ gcache::Page::Page (void* ps, const std::string& name, size_t size, int dbg, boo
     size_ (mmap_.get_size()),
     min_space_ (space_),
 #endif /* PXC */
-    debug_(dbg)
+    debug_(dbg),
+    closed_(false)
 {
     log_info << "Created page " << name << " of size " << space_
              << " bytes";
@@ -89,7 +97,7 @@ gcache::Page::malloc (size_type size)
 {
     Limits::assert_size(size);
 
-    if (size <= space_)
+    if (size <= space_ && !closed_)
     {
         BufferHeader* bh(BH_cast(next_));
 
@@ -120,7 +128,8 @@ gcache::Page::malloc (size_type size)
 
         assert (next_ <= static_cast<uint8_t*>(mmap_.get_ptr()) + mmap_.get_size());
 
-        if (debug_) { log_info << name() << " allocd " << bh; }
+        if (debug_) { log_info << name() << " allocd " << bh << ", used: "
+                               << used_ << ", mapped: " << mapped_; }
 #endif
 
         return (bh + 1);
@@ -138,6 +147,7 @@ void*
 gcache::Page::realloc (void* ptr, size_type size)
 {
     Limits::assert_size(size);
+    if (closed_) return nullptr;
 
     BufferHeader* bh(ptr2BH(ptr));
 
@@ -206,8 +216,8 @@ size_t gcache::Page::allocated_pool_size ()
 
 void gcache::Page::print(std::ostream& os) const
 {
-    os << "page file: " << name() << ", size: " << size() << ", used: "
-       << used_;
+    os << "name: " << name() << ", size: " << size() << ", used: " << used()
+       << ", mapped: " << mapped_ << ", seqno_max: " << seqno_max();
 
     if (used_ > 0 && debug_ > 0)
     {
