@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2024 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2025 Codership Oy <info@codership.com>
  */
 
 /*! @file page file class */
@@ -7,6 +7,7 @@
 #ifndef _gcache_page_hpp_
 #define _gcache_page_hpp_
 
+#include "gcache_seqno.hpp"
 #include "gcache_memops.hpp"
 #include "gcache_bh.hpp"
 
@@ -24,7 +25,13 @@ namespace gcache
     {
     public:
 
-        Page (void* ps, const std::string& name, size_t size, int dbg, bool encrypt, size_t encrypt_cache_page_size, size_t encrypt_cache_pages_count);
+        Page (void*              ps,
+              const std::string& name,
+              size_t             size,
+              int                dbg,
+              bool               encrypt,
+              size_t             encrypt_cache_page_size,
+              size_t encrypt_cache_pages_count);
         ~Page () {}
 
         void* malloc  (size_type size);
@@ -40,10 +47,12 @@ namespace gcache
             assert(bh->size > 0);
             assert(bh->store == BUFFER_IN_PAGE);
             assert(bh->ctx == reinterpret_cast<BH_ctx_t>(this));
-            assert (used_ > 0);
+            assert(!closed_);
+            assert(used_ > 0);
             used_--;
 #ifndef NDEBUG
-            if (debug_) { log_info << name() << " freed " << bh; }
+            if (debug_) { log_info << name() << " freed " << bh << ", used: "
+                                   << used_ << ", mapped: " << mapped_; }
 #endif
         }
 
@@ -56,20 +65,37 @@ namespace gcache
             assert(bh->store == BUFFER_IN_PAGE);
             assert(bh->ctx == reinterpret_cast<BH_ctx_t>(this));
             assert(BH_is_released(bh)); // will be marked unreleased by caller
+            assert(!closed_); // minimum available seqno must be adjusted
+                              // before closing the page
             used_++;
 #ifndef NDEBUG
-            if (debug_) { log_info << name() << " repossessed " << bh; }
+            if (debug_) { log_info << name() << " repossessed " << bh
+                                   << ", used: " << used_ << ", mapped: "
+                                   << mapped_; }
 #endif
         }
 
         void discard (BufferHeader* bh)
         {
+            assert(bh >= mmap_.get_ptr());
+            assert(reinterpret_cast<uint8_t*>(bh) + bh->size <= next_);
+            assert(bh->size > 0);
+            assert(bh->seqno_g != SEQNO_NONE);
+            assert(bh->store == BUFFER_IN_PAGE);
+            assert(bh->ctx == reinterpret_cast<BH_ctx_t>(this));
+            assert(BH_is_released(bh));
+            assert(mapped_ > 0 || bh->seqno_g == SEQNO_ILL);
+            mapped_ -= (bh->seqno_g != SEQNO_ILL);
 #ifndef NDEBUG
-            if (debug_) { log_info << name() << " discarded " << bh; }
+            if (bh->seqno_g != SEQNO_ILL && 0 == mapped_)
+                assert(seqno_max_ == bh->seqno_g);
+            if (debug_) { log_info << name() << " discarded " << bh
+                                   << ", used: " << used_ << ", mapped: "
+                                   << mapped_; }
 #endif
         }
 
-        size_t used () const { return used_; }
+        size_t used() const { return used_; }
 
 #ifdef PXC
         size_t size() const { return size_; } /* size on storage */
@@ -85,6 +111,29 @@ namespace gcache
 
         void  seqno_unlock() {}
 
+        void  seqno_assign(seqno_t const seqno)
+        {
+            assert(seqno > 0);
+            assert(used_ > 0); // cannot assign seqno to unused buffer
+            assert(!closed_);  // cannot be closed while used
+            seqno_max_ = std::max(seqno_max_, seqno);
+            mapped_++;
+#ifndef NDEBUG
+            if (debug_) { log_info << name() << " seqno_assign(" << seqno
+                                   << ") seqno_max: " << seqno_max_
+                                   << ", used: " << used_ << ", mapped: "
+                                   << mapped_; }
+#endif
+        }
+
+        seqno_t seqno_max() const { return seqno_max_; }
+
+        void close()
+        {
+            assert(0 == used_);
+            closed_ = true;
+        }
+
         /* Drop filesystem cache on the file */
         void drop_fs_cache() const;
 
@@ -103,15 +152,18 @@ namespace gcache
         gu::FileDescriptor fd_;
         std::shared_ptr<gu::IMMap>  mmapptr_;  // keep mmap_ member as the reference
         gu::IMMap&         mmap_;
+        seqno_t            seqno_max_; // highest seqno assigned to buffer
         void* const        ps_;
         uint8_t*           next_;
         size_t             space_;
         size_t             used_;
+        size_t             mapped_; // buffers mapped in seqno2ptr map
 #ifdef PXC
         size_t             size_;
         size_t             min_space_;
 #endif /* PXC */
         int                debug_;
+        bool               closed_; // page not available any more
 
         Page(const gcache::Page&);
         Page& operator=(const gcache::Page&);
