@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2018 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2025 Codership Oy <info@codership.com>
  */
 
 /*! @file page store implementation */
@@ -47,11 +47,9 @@ make_page_name (const std::string& base_name, size_t count)
     return os.str();
 }
 
-static void*
-remove_file (void* __restrict__ arg)
+static void
+remove_file (const std::string& file_name)
 {
-    char* const file_name (static_cast<char*>(arg));
-
 #ifdef PXC
 #ifdef HAVE_PSI_INTERFACE
     pfs_instr_callback(WSREP_PFS_INSTR_TYPE_THREAD,
@@ -61,26 +59,67 @@ remove_file (void* __restrict__ arg)
 #endif /* HAVE_PSI_INTERFACE */
 #endif /* PXC */
 
-    if (NULL != file_name)
+    if (file_name.length() > 0)
     {
-        if (remove (file_name))
+        if (::remove(file_name.c_str()))
         {
             int err = errno;
 
-            log_error << "Failed to remove page file '" << file_name << "': "
-                      << err << " (" << strerror(err) << ")";
+            log_error << "Failed to remove page file '" << file_name
+                      << "': " << err << " (" << strerror(err) << ")";
         }
         else
         {
             log_info << "Deleted page " << file_name;
         }
-
-        free (file_name);
     }
     else
     {
-        log_error << "Null file name in " << __FUNCTION__;
+        log_error << "Empty file name in " << __FUNCTION__;
     }
+}
+
+struct delete_thread_arg
+{
+    gcache::SeqnoMap& seqno_map_;
+    gcache::Page&     page_;
+    pthread_t         previous_thread_;
+    bool              debug_;
+
+    delete_thread_arg(gcache::SeqnoMap& m, gcache::Page& p, pthread_t t, bool d)
+        :
+        seqno_map_(m),
+        page_     (p),
+        previous_thread_(t),
+        debug_    (d)
+    {}
+    ~delete_thread_arg() { delete &page_; }
+};
+
+static void*
+discard_page(void* __restrict__ a)
+{
+    delete_thread_arg* arg(static_cast<delete_thread_arg*>(a));
+
+    auto& page(arg->page_);
+
+#ifndef NDEBUG
+    if (arg->debug_) { log_info << "PageStore::discard_page() prev. thread: "
+                                << arg->previous_thread_ << ", page: "
+                                << page; }
+#endif
+
+    if (arg->previous_thread_ != pthread_t(-1))
+        pthread_join(arg->previous_thread_, NULL);
+
+    if (page.seqno_max() > 0)
+        arg->seqno_map_.seqno_discard(page.seqno_max());
+
+    std::string const file_name(page.name());
+
+    delete arg;
+
+    remove_file(file_name);
 
 #ifdef PXC
 #ifdef HAVE_PSI_INTERFACE
@@ -94,11 +133,9 @@ remove_file (void* __restrict__ arg)
     pthread_exit(NULL);
 }
 
-/*
- * Returns false if there are no more pages to be deleted (either
- * the queue is empty or if the first page is in use).
- * Otherwise, returns true.
-*/
+/* This method does minimum work while holding global lock and then
+ * delegates seqno2ptr map cleanup to a dedicated thread. If there is a
+ * previously launched thread it will be joined by the new one. */
 bool
 gcache::PageStore::delete_page ()
 {
@@ -108,44 +145,40 @@ gcache::PageStore::delete_page ()
 
     Page* const page = pages_.front();
 
-    if (page->used() > 0) return false;
+#ifndef NDEBUG
+    if (debug_) { log_info << "PageStore::delete_page() " << *page; }
+#endif
+
+    if (page->used() > 0 || page->seqno_max() >= seqno_locked_)
+        return false;
 
     pages_.pop_front();
-
-    char* const file_name(strdup(page->name().c_str()));
-
     total_size_ -= page->size();
-
     if (current_ == page) current_ = 0;
 
-    delete page;
+    /* While we are still holding global lock close the page and up the
+     * low available limit to the max seqno contained in a page */
+    seqno_map_.set_low_limit(page->seqno_max());
+    page->close();
 
-#ifdef GCACHE_DETACH_THREAD
-    pthread_t delete_thr_;
-#else
-    if (delete_thr_ != pthread_t(-1)) pthread_join (delete_thr_, NULL);
-#endif /* GCACHE_DETACH_THERAD */
-
-#ifdef PXC
-    int err = gu_thread_create (&delete_thr_, &delete_page_attr_, remove_file,
-                                file_name);
-#else
-    int err = pthread_create (&delete_thr_, &delete_page_attr_, remove_file,
-                              file_name);
-#endif /* PXC */
+    /* if there is currently another thread running it will be joined in
+     * this new thread */
+    pthread_t const saved(delete_thr_);
+    int err = pthread_create(&delete_thr_, &delete_page_attr_, discard_page,
+                             new delete_thread_arg(seqno_map_, *page,
+                                                   delete_thr_, debug_));
     if (0 != err)
     {
-        delete_thr_ = pthread_t(-1);
-        gu_throw_system_error(err)
-            << "Failed to create page file deletion thread";
+        delete_thr_ = saved;
+        gu_throw_system_error(err) << "Failed to create page deletion thread";
     }
 
     return true;
 }
 
 /* Deleting pages only from the beginning kinda means that some free pages
- * can be locked in the middle for a while. Leaving it like that for simplicity
- * for now. */
+ * can be locked in the middle for a while. Leaving it like that for
+ * simplicity for now. */
 void
 gcache::PageStore::cleanup ()
 {
@@ -187,6 +220,16 @@ gcache::PageStore::cleanup ()
 #endif /* PXC */
 }
 
+void
+gcache::PageStore::wait_page_discard() const
+{
+    if (delete_thr_ != pthread_t(-1))
+    {
+        pthread_join(delete_thr_, NULL);
+        delete_thr_ = pthread_t(-1);
+    }
+}
+
 void
 gcache::PageStore::reset ()
 {
@@ -196,9 +239,9 @@ gcache::PageStore::reset ()
 inline void
 gcache::PageStore::new_page (size_type size)
 {
-    Page* const page(new Page
-                     (this, make_page_name (base_name_, count_), size, debug_,
-                      encrypt_, encrypt_cache_page_size_, encrypt_cache_size_));
+    Page* const page(new Page(this, make_page_name (base_name_, count_), size,
+                              debug_, encrypt_, encrypt_cache_page_size_,
+                              encrypt_cache_size_));
 
     pages_.push_back (page);
     total_size_ += page->size();
@@ -206,7 +249,8 @@ gcache::PageStore::new_page (size_type size)
     count_++;
 }
 
-gcache::PageStore::PageStore (const std::string& dir_name,
+gcache::PageStore::PageStore (SeqnoMap&          seqno_map,
+                              const std::string& dir_name,
                               size_t             keep_size,
                               size_t             page_size,
                               int                dbg,
@@ -215,7 +259,9 @@ gcache::PageStore::PageStore (const std::string& dir_name,
                               size_t             encrypt_cache_page_size,
                               size_t             encrypt_cache_size)
     :
+    seqno_map_ (seqno_map),
     base_name_ (make_base_name(dir_name)),
+    seqno_locked_(SEQNO_MAX),
     keep_size_ (keep_size),
     page_size_ (page_size),
     keep_page_ (keep_page),
@@ -225,9 +271,7 @@ gcache::PageStore::PageStore (const std::string& dir_name,
     total_size_(0),
     delete_page_attr_(),
     debug_     (dbg & DEBUG)
-#ifndef GCACHE_DETACH_THREAD
     , delete_thr_(pthread_t(-1))
-#endif /* GCACHE_DETACH_THREAD */
     , encrypt_(encrypt)
     , encrypt_cache_page_size_(encrypt_cache_page_size)
     , encrypt_cache_size_(encrypt_cache_size)
@@ -236,20 +280,9 @@ gcache::PageStore::PageStore (const std::string& dir_name,
 
     if (0 != err)
     {
-        gu_throw_system_error(err) << "Failed to initialize page file deletion "
-                                   << "thread attributes";
+        gu_throw_system_error(err) << "Failed to initialize page file "
+            "deletion thread attributes";
     }
-
-#ifdef GCACHE_DETACH_THREAD
-    err = pthread_attr_setdetachstate (&delete_page_attr_,
-                                       PTHREAD_CREATE_DETACHED);
-    if (0 != err)
-    {
-        pthread_attr_destroy (&delete_page_attr_);
-        gu_throw_system_error(err) << "Failed to set DETACHED attribute to "
-                                   << "page file deletion thread";
-    }
-#endif /* GCACHE_DETACH_THREAD */
 }
 
 gcache::PageStore::~PageStore ()
@@ -257,9 +290,8 @@ gcache::PageStore::~PageStore ()
     try
     {
         while (pages_.size() && delete_page()) {};
-#ifndef GCACHE_DETACH_THREAD
+
         if (delete_thr_ != pthread_t(-1)) pthread_join (delete_thr_, NULL);
-#endif /* GCACHE_DETACH_THREAD */
     }
     catch (gu::Exception& e)
     {
@@ -295,9 +327,7 @@ gcache::PageStore::malloc_new (size_type size)
     }
     catch (gu::Exception& e)
     {
-        log_error << "Cannot create new cache page: "
-                  << e.what();
-        // abort();
+        log_error << "Cannot create new cache page: " << e.what();
     }
 
     return ret;
