@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2021 Codership Oy <info@codership.com>
+ * Copyright (C) 2008-2024 Codership Oy <info@codership.com>
  *
  * $Id$
  */
@@ -36,6 +36,7 @@
  *
  */
 
+#include <inttypes.h> // PRId64
 #define GCS_STATE_MSG_ACCESS
 #include "../gcs_core.hpp"
 #include "../gcs_dummy.hpp"
@@ -656,7 +657,7 @@ CORE_TEST_OWN (int gcs_proto_ver)
     Cache->free(act_r.out);
     ck_assert(!CORE_RECV_ACT (&act_r, act_buf, act_size, GCS_ACT_WRITESET));
     ck_assert_msg(-ENOTCONN == act_r.seqno,
-                  "Expected -ENOTCONN, received %ld (%s)",
+                  "Expected -ENOTCONN, received %" PRId64 " (%s)",
                   act_r.seqno, strerror (-act_r.seqno));
 
     /*
@@ -703,7 +704,7 @@ CORE_TEST_OWN (int gcs_proto_ver)
     Cache->free(act_r.out);
     ck_assert(!CORE_RECV_ACT (&act_r, act_buf, act_size, GCS_ACT_WRITESET));
     ck_assert_msg(-ENOTCONN == act_r.seqno,
-                  "Expected -ENOTCONN, received %ld (%s)",
+                  "Expected -ENOTCONN, received %" PRId64 " (%s)",
                   act_r.seqno, strerror (-act_r.seqno));
 
     /*
@@ -722,7 +723,7 @@ CORE_TEST_OWN (int gcs_proto_ver)
     ck_assert(!CORE_SEND_END (&act_s, act_size));
     ck_assert(!CORE_RECV_ACT (&act_r, act_buf, act_size, GCS_ACT_WRITESET));
     ck_assert_msg(-ERESTART == act_r.seqno,
-                  "Expected -ERESTART, received %ld (%s)",
+                  "Expected -ERESTART, received %" PRId64 " (%s)",
                   act_r.seqno, strerror (-act_r.seqno));
 
     /*
