@@ -35,6 +35,7 @@
 # encryption specific variables.
 encrypt=0
 xbstream_opts=""
+lock_ddl_opt=""
 
 nproc=1
 ecode=0
@@ -401,7 +402,7 @@ check_for_dhparams()
                 wsrep_log_error "Please ensure that openssl is installed and is in the path"
                 wsrep_log_error "Line $LINENO"
                 wsrep_log_error "****************************************************** "
-                exit 2
+                safe_exit 2
             fi
             wsrep_log_info "Could not find dhparams file, creating $DATA/dhparams.pem"
 
@@ -411,7 +412,7 @@ check_for_dhparams()
                 wsrep_log_error "* Could not create the dhparams.pem file with OpenSSL. "
                 wsrep_log_error "* Line $LINENO"
                 wsrep_log_error "****************************************************** "
-                exit 22
+                safe_exit 22
             fi
         fi
         ssl_dhparams="$DATA/dhparams.pem"
@@ -438,7 +439,7 @@ verify_cert_matches_key()
         wsrep_log_error "Please ensure that openssl and diff are installed and in the path"
         wsrep_log_error "Line $LINENO"
         wsrep_log_error "****************************************************** "
-        exit 2
+        safe_exit 2
     fi
 
     # generate the public key from the cert and the key
@@ -450,7 +451,7 @@ verify_cert_matches_key()
         wsrep_log_error "* Please check your certificate and key files. "
         wsrep_log_error "* Line $LINENO"
         wsrep_log_error "****************************************************** "
-        exit 22
+        safe_exit 22
     fi
 }
 
@@ -473,7 +474,7 @@ verify_ca_matches_cert()
         wsrep_log_error "Please ensure that openssl is installed and is in the path"
         wsrep_log_error "Line $LINENO"
         wsrep_log_error "****************************************************** "
-        exit 2
+        safe_exit 2
     fi
 
     if ! openssl verify -verbose -CAfile "$ca_path" "$cert_path" >/dev/null  2>&1
@@ -484,7 +485,7 @@ verify_ca_matches_cert()
         wsrep_log_error "* Please check your certificate and CA files.                   "
         wsrep_log_error "* Line $LINENO"
         wsrep_log_error "*************************************************************** "
-        exit 22
+        safe_exit 22
     fi
 
 }
@@ -515,7 +516,7 @@ verify_file_exists()
         fi
 
         wsrep_log_error "****************************************************** "
-        exit 22
+        safe_exit 22
     fi
 }
 
@@ -535,7 +536,7 @@ get_transfer()
             wsrep_log_error "nc(netcat) not found in path: $PATH"
             wsrep_log_error "Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 2
+            safe_exit 2
         fi
 
         if [[ $encrypt -eq 4 ]]; then
@@ -544,7 +545,7 @@ get_transfer()
             wsrep_log_error "* is not supported when using nc(netcat)."
             wsrep_log_error "* Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 22
+            safe_exit 22
         fi
 
         wsrep_log_debug "Using netcat as streamer"
@@ -579,7 +580,7 @@ get_transfer()
             wsrep_log_error "socat not found in path: $PATH"
             wsrep_log_error "Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 2
+            safe_exit 2
         fi
 
         donor_extra=""
@@ -603,7 +604,7 @@ get_transfer()
                 wsrep_log_error "* Unable to encrypt SST communications. "
                 wsrep_log_error "* Line $LINENO"
                 wsrep_log_error "****************************************************** "
-                exit 2
+                safe_exit 2
             fi
 
             # Determine the socat version
@@ -613,7 +614,7 @@ get_transfer()
                 wsrep_log_error "* Cannot determine the socat version. "
                 wsrep_log_error "* Line $LINENO"
                 wsrep_log_error "****************************************************** "
-                exit 2
+                safe_exit 2
             fi
 
             # Convert the dhparams path into an absolute path
@@ -836,6 +837,7 @@ read_cnf()
     fi
 
     xbstream_opts=$(parse_cnf sst xbstream-opts "")
+    lock_ddl_opt=$(parse_cnf xtrabackup lock-ddl "")
 }
 
 #
@@ -903,7 +905,7 @@ get_stream()
             wsrep_log_error "Please verify that PXC was installed correctly."
             wsrep_log_error "* Line $LINENO"
             wsrep_log_error "******************************************** "
-            exit 2
+            safe_exit 2
         fi
 
         # It's ok to use any xbstream, it's compatible across versions
@@ -919,7 +921,7 @@ get_stream()
             wsrep_log_error "tar was not found in PATH! Make sure you have it installed."
             wsrep_log_error "* Line $LINENO"
             wsrep_log_error "******************************************** "
-            exit 2
+            safe_exit 2
         fi
 
         sfmt="tar"
@@ -1206,7 +1208,7 @@ wait_for_listen()
         wsrep_log_error "******** FATAL ERROR *********************** "
         wsrep_log_error "* Cannot find /proc/$$/net/tcp (or tcp6)"
         wsrep_log_error "******************************************** "
-        exit 1
+        safe_exit 1
     fi
     for i in "${!header[@]}"; do
         if [[ ${header[$i]} = "local_address" ]]; then
@@ -1221,7 +1223,7 @@ wait_for_listen()
         wsrep_log_error "* Unexpected /proc/xx/net/tcp layout: cannot find local_address"
         wsrep_log_error "* Line $LINENO"
         wsrep_log_error "******************************************** "
-        exit 1
+        safe_exit 1
     fi
 
     wsrep_log_debug "$LINENO: local_address index is $ip_index"
@@ -1368,7 +1370,7 @@ recv_data_from_donor_to_joiner()
         wsrep_log_error "******************* FATAL ERROR ********************** "
         wsrep_log_error "SST script interrupted"
         wsrep_log_error "****************************************************** "
-        exit 32
+        safe_exit 32
     fi
 
     if [[ ${RC[0]} -eq 124 ]]; then
@@ -1382,7 +1384,7 @@ recv_data_from_donor_to_joiner()
         wsrep_log_error "Check the network connection and restart the joiner node."
         wsrep_log_error "Line $LINENO"
         wsrep_log_error "****************************************************** "
-        exit 32
+        safe_exit 32
     fi
 
     # Here we know that transfer pipeline received SIGKILL
@@ -1396,7 +1398,7 @@ recv_data_from_donor_to_joiner()
         wsrep_log_error "exit codes: ${RC[@]}"
         wsrep_log_error "Line $LINENO"
         wsrep_log_error "****************************************************** "
-        exit 32
+        safe_exit 32
     fi
 
     for ecode in "${RC[@]}"; do
@@ -1406,7 +1408,7 @@ recv_data_from_donor_to_joiner()
                             "exit codes: ${RC[@]}"
             wsrep_log_error "Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 32
+            safe_exit 32
         fi
     done
 
@@ -1423,7 +1425,7 @@ recv_data_from_donor_to_joiner()
         wsrep_log_error "****************************************************** "
         wsrep_log_debug "Contents of datadir"
         wsrep_log_debug "$(ls -l ${dir}/*)"
-        exit 32
+        safe_exit 32
     fi
 }
 
@@ -1448,7 +1450,7 @@ send_data_from_donor_to_joiner()
         wsrep_log_error "Could not find/read the file: $FILE_TO_STREAM"
         wsrep_log_error "Line $LINENO"
         wsrep_log_error "****************************************************** "
-        exit 2
+        safe_exit 2
     fi
 
     set +e
@@ -1481,7 +1483,7 @@ send_data_from_donor_to_joiner()
                             "exit codes: ${RC[@]}"
             wsrep_log_error "Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 32
+            safe_exit 32
         fi
     done
 }
@@ -1507,15 +1509,15 @@ initialize_tmpdir()
     if [[ -n "${tmpdir_path}" ]]; then
         if [[ ! -d "${tmpdir_path}" ]]; then
             wsrep_log_error "Cannot find the directory, ${tmpdir_path}, the tmpdir must exist before startup."
-            exit 2
+            safe_exit 2
         fi
         if [[ ! -r "${tmpdir_path}" ]]; then
             wsrep_log_error "The temporary directory, ${tmpdir_path}, is not readable.  Please check the directory permissions."
-            exit 22
+            safe_exit 22
         fi
         if [[ ! -w "${tmpdir_path}" ]]; then
             wsrep_log_error "The temporary directory, ${tmpdir_path}, is not writable.  Please check the directory permissions."
-            exit 22
+            safe_exit 22
         fi
     fi
 
@@ -1590,6 +1592,7 @@ function initialize_pxb_commands()
 
     local disver=""
     local pxb_root pxb_bin_path pxb_plugin_dir
+    local lock_ddl="--lock-ddl"
 
     # We need to use PXB compatible with donor
     pxb_root="${XTRABACKUP_PATH_PREFIX}${donor_version_str}"
@@ -1606,20 +1609,41 @@ function initialize_pxb_commands()
         wsrep_log_error "Unable to run xtrabackup (does not exist): $pxb_bin_path"
         wsrep_log_error "Line $LINENO"
         wsrep_log_error "****************************************************** "
-        exit 2
+        safe_exit 2
     fi
     if ! [[ -x $pxb_bin_path ]]; then
         wsrep_log_error "******************* FATAL ERROR ********************** "
         wsrep_log_error "Unable to run xtrabackup (non-executable): $pxb_bin_path"
         wsrep_log_error "Line $LINENO"
         wsrep_log_error "****************************************************** "
-        exit 2
+        safe_exit 2
     fi
 
     if ${pxb_bin_path} --help 2>/dev/null | grep -q -- '--version-check'; then
         disver="--no-version-check"
     fi
 
+    #
+    # --lock-ddl option
+    #
+    # Use user-provided option if set
+    [[ -n "$lock_ddl_opt" ]] && lock_ddl="--lock-ddl=$lock_ddl_opt"
+
+    # Check if xtrabackup (PXB) Pro version is used (supports --lock-ddl=REDUCED)
+    if ${pxb_bin_path} --version 2>&1 | grep -q -- '-pro'; then
+        if [[ -z "$lock_ddl_opt" ]]; then
+            lock_ddl="--lock-ddl=REDUCED"
+            wsrep_log_info "PXB Pro detected; using default --lock-ddl=REDUCED for SST"
+        fi
+    elif [[ "${lock_ddl_opt^^}" == "REDUCED" ]]; then
+        # If REDUCED is requested but not supported, exit with error
+        wsrep_log_error "******************* FATAL ERROR **********************"
+        wsrep_log_error "The xtrabackup version does not support --lock-ddl=REDUCED option."
+        wsrep_log_error "Line $LINENO"
+        wsrep_log_error "******************************************************"
+        safe_exit 2
+    fi
+
     local xb_version=$(${pxb_bin_path} --version 2>&1 | grep -oe ' [0-9]\.[0-9][\.0-9]*' | head -n1)
     xb_version=${xb_version# }
     wsrep_log_debug "pxb-version:$xb_version"
@@ -1678,7 +1702,7 @@ function initialize_pxb_commands()
                 --target-dir=\${DATA} 2>&1 | logger -p daemon.err -t ${ssystag}innobackupex-move "
             INNOBACKUP="${pxb_bin_path} --defaults-file=${WSREP_SST_OPT_CONF} \
                 --defaults-group=mysqld${WSREP_SST_OPT_CONF_SUFFIX} $disver $iopts \
-                \$INNOEXTRA \$keyringbackupopt --lock-ddl --backup --galera-info \
+                \$INNOEXTRA \$keyringbackupopt $lock_ddl --backup --galera-info \
                 \$encrypt_backup_options --stream=\$sfmt \
                 --xtrabackup-plugin-dir="$pxb_plugin_dir" \
                 --target-dir=\$itmpdir 2> >(logger -p daemon.err -t ${ssystag}innobackupex-backup)"
@@ -1701,7 +1725,7 @@ function initialize_pxb_commands()
             --target-dir=\${DATA} &>\${DATA}/innobackup.move.log"
         INNOBACKUP="${pxb_bin_path} --defaults-file=${WSREP_SST_OPT_CONF} \
             --defaults-group=mysqld${WSREP_SST_OPT_CONF_SUFFIX} $disver $iopts \
-            \$INNOEXTRA \$keyringbackupopt --lock-ddl --backup --galera-info \
+            \$INNOEXTRA \$keyringbackupopt $lock_ddl --backup --galera-info \
             \$encrypt_backup_options --stream=\$sfmt \
             --xtrabackup-plugin-dir="$pxb_plugin_dir" \
             --target-dir=\$itmpdir 2>\${DATA}/innobackup.backup.log"
@@ -1722,7 +1746,7 @@ function verify_pxb_version()
         wsrep_log_error "    Expected location: $pxb_bin_path"
         wsrep_log_error "Please verify that PXC was installed correctly."
         wsrep_log_error "****************************************************** "
-        exit 2
+        safe_exit 2
     fi
 
     local xb_version=$($pxb_bin_path --version 2>&1 | grep -oe ' [0-9]\.[0-9]\.[0-9]*' | head -n1)
@@ -1734,7 +1758,7 @@ function verify_pxb_version()
         wsrep_log_error "$pxb_bin_path"
         wsrep_log_error "Please verify that PXC was installed correctly."
         wsrep_log_error "****************************************************** "
-        exit 2
+        safe_exit 2
     fi
 
     if compare_versions "$xb_version" "<" "$pxb_expected_version"; then
@@ -1743,7 +1767,7 @@ function verify_pxb_version()
         wsrep_log_error "xtrabackup-$pxb_expected_version or higher is needed to perform an SST"
         wsrep_log_error "$pxb_bin_path"
         wsrep_log_error "****************************************************** "
-        exit 2
+        safe_exit 2
     fi
 }
 
@@ -1759,7 +1783,7 @@ if [[ -z $MYSQL_VERSION ]]; then
     wsrep_log_error "******************* FATAL ERROR ********************** "
     wsrep_log_error "FATAL: Cannot determine the mysqld server version"
     wsrep_log_error "****************************************************** "
-    exit 2
+    safe_exit 2
 fi
 
 # Verify PXB versions we have
@@ -1779,13 +1803,13 @@ if [[ ! ${WSREP_SST_OPT_ROLE} == 'joiner' && ! ${WSREP_SST_OPT_ROLE} == 'donor'
     wsrep_log_error "Invalid role ${WSREP_SST_OPT_ROLE}"
     wsrep_log_error "Line $LINENO"
     wsrep_log_error "****************************************************** "
-    exit 22
+    safe_exit 22
 fi
 
 #
 # read configuration and setup ports for streaming data.
 read_variables_from_stdin
-[[ $? -ne 0 ]] && exit 2
+[[ $? -ne 0 ]] && safe_exit 2
 
 #
 # Only the DONOR is sent the credentials
@@ -1794,7 +1818,7 @@ if [[ $WSREP_SST_OPT_ROLE == "donor" ]]; then
         wsrep_log_error "******************* FATAL ERROR ********************** "
         wsrep_log_error "FATAL: The required auth credentials for an SST have not been received"
         wsrep_log_error "****************************************************** "
-        exit 2
+        safe_exit 2
     fi
 fi
 
@@ -1824,7 +1848,7 @@ if [[ "$pxc_encrypt_cluster_traffic" == "on" ]]; then
             wsrep_log_error "* Please specify a CA file with the 'ssl-ca' option. "
             wsrep_log_error "* Line $LINENO"
             wsrep_log_error "**************************************************** "
-            return 2
+            save_exit 2
         fi
     fi
     if [[ -z "$ssl_cert" ]]; then
@@ -1836,7 +1860,7 @@ if [[ "$pxc_encrypt_cluster_traffic" == "on" ]]; then
             wsrep_log_error "* Please specify a certificate file with the 'ssl-cert' option. "
             wsrep_log_error "* Line $LINENO"
             wsrep_log_error "****************************************************** "
-            return 2
+            safe_exit 2
         fi
     fi
     if [[ -z "$ssl_key" ]]; then
@@ -1848,7 +1872,7 @@ if [[ "$pxc_encrypt_cluster_traffic" == "on" ]]; then
             wsrep_log_error "* Please specify a key file with the 'ssl-key' option. "
             wsrep_log_error "* Line $LINENO"
             wsrep_log_error "****************************************************** "
-            return 2
+            safe_exit 2
         fi
     fi
 
@@ -1864,7 +1888,7 @@ if [[ $encrypt -eq 1 || $encrypt -eq 2 || $encrypt -eq 3 ]]; then
     wsrep_log_error "* Please use SSL-based encryption (encrypt=4)"
     wsrep_log_error "* Line $LINENO"
     wsrep_log_error "****************************************************** "
-    exit 22
+    safe_exit 22
 fi
 
 
@@ -1880,7 +1904,7 @@ if $MY_PRINT_DEFAULTS -c $WSREP_SST_OPT_CONF xtrabackup | grep -q encrypt; then
     wsrep_log_error "* SSL-based encryption (encrypt=4)."
     wsrep_log_error "* Line $LINENO"
     wsrep_log_error "****************************************************** "
-    exit 2
+    safe_exit 2
 fi
 
 
@@ -1933,7 +1957,7 @@ then
                         "is unencrypted. Enable encryption for SST traffic"
         wsrep_log_error "Line $LINENO"
         wsrep_log_error "****************************************************** "
-        exit 22
+        safe_exit 22
     fi
 
     # Create the SST info file
@@ -1967,7 +1991,7 @@ then
             wsrep_log_error "The joiner is not supported for this version of donor"
             wsrep_log_error "Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 93
+            safe_exit 93
         fi
 
         # main temp directory for xtrabackup (target-dir)
@@ -2113,7 +2137,7 @@ then
         done
 
         if [[ $do_exit -eq 1 ]]; then
-            exit 22
+            safe_exit 22
         fi
 
     else # BYPASS FOR IST
@@ -2169,6 +2193,15 @@ then
     # May need xtrabackup_checkpoints later on
     rm -f ${DATA}/xtrabackup_binary ${DATA}/xtrabackup_galera_info  ${DATA}/xtrabackup_logfile
 
+    # After starting socat listener in recv_data_from_donor_to_joiner sst-info,
+    # wait_for_listen will send "ready" message to the application.
+    # Application forms SST request from "ready" data and marks grastate.dat
+    # as 'unsafe'. From now till the point when we delete data directory,
+    # grastate.dat can be turned back to 'safe' state. This will prevent forced
+    # SST in case something goes wrong before datadir deletion.
+    # Fail "soft" until we delete datadir
+    SAFE_EXIT_CODE_OVERRIDE=${EAGAIN}
+
     # Note: this is started as a background process
     # So it has to wait for processes that are started by THIS process
     wait_for_listen $$ ${WSREP_SST_OPT_HOST} ${WSREP_SST_OPT_PORT:-4444} ${MODULE} &
@@ -2216,7 +2249,7 @@ then
         wsrep_log_error "Did not receive expected file from donor: '${SST_INFO_FILE}'"
         wsrep_log_error "Line $LINENO"
         wsrep_log_error "****************************************************** "
-        exit 32
+        safe_exit 32
     fi
 
     if [ ! -r "${STATDIR}/${IST_FILE}" ]
@@ -2253,7 +2286,7 @@ then
             wsrep_log_error "${local_version_str} (the same version as this node)."
             wsrep_log_error "Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 2
+            safe_exit 2
         fi
 
         # Is this node's pxc version < donor's pxc version?
@@ -2264,7 +2297,7 @@ then
             wsrep_log_error "Upgrade this node before joining the cluster."
             wsrep_log_error "Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 2
+            safe_exit 2
         fi
 
         # Initializes the command-line args for XB
@@ -2282,7 +2315,7 @@ then
                 wsrep_log_error "Please remove it or specify alternative temporary directory location by setting [sst]/tmpdir"
                 wsrep_log_error "Line $LINENO"
                 wsrep_log_error "****************************************************** "
-                exit 2
+                safe_exit 2
             fi
             mkdir -p ${DATA}/sst-xb-tmpdir
             JOINER_SST_DIR=$DATA/sst-xb-tmpdir
@@ -2299,7 +2332,7 @@ then
                             "but DONOR is not"
             wsrep_log_error "Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 32
+            safe_exit 32
 
         fi
         # server-id is already part of backup-my.cnf so avoid appending it.
@@ -2314,7 +2347,7 @@ then
                             "(file/vault) but JOINER is not"
             wsrep_log_error "Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 32
+            safe_exit 32
         fi
 
         if [[ -n $transition_key ]]; then
@@ -2335,7 +2368,7 @@ then
             wsrep_log_error "Parent mysqld process (PID:${WSREP_SST_OPT_PARENT}) terminated unexpectedly."
             wsrep_log_error "Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 32
+            safe_exit 32
         fi
 
         get_stream
@@ -2343,6 +2376,13 @@ then
             strmcmd=" $sdecomp | $strmcmd"
         fi
 
+        # Up to this point, if anything happened that prevented SST (eg. network
+        # issue, wrong keyring configuration), datadir was not touched, so we should
+        # leave the node as it was (grastate.dat)
+        # From now on, there is no way back: we will receive SST, or the node won't
+        # work.
+        SAFE_EXIT_CODE_OVERRIDE=
+
         (recv_data_from_donor_to_joiner "$JOINER_SST_DIR" "${stagemsg}-SST" 0 0) &
         jpid=$!
         wsrep_log_info "Proceeding with SST........."
@@ -2389,7 +2429,7 @@ then
             wsrep_log_error "xtrabackup_checkpoints missing. xtrabackup/SST failed on DONOR. Check DONOR log"
             wsrep_log_error "Line $LINENO"
             wsrep_log_error "****************************************************** "
-            exit 2
+            safe_exit 2
         fi
 
         # Rebuild indexes for compact backups
@@ -2413,7 +2453,7 @@ then
                 wsrep_log_error "qpress not found in path: $PATH"
                 wsrep_log_error "Line $LINENO"
                 wsrep_log_error "****************************************************** "
-                exit 22
+                safe_exit 22
             fi
 
             if [[ -n $progress ]] && pv --help | grep -q 'line-mode'; then
@@ -2452,7 +2492,7 @@ then
                 wsrep_log_error "Decompression failed. Exit code: $extcode"
                 wsrep_log_error "Line $LINENO"
                 wsrep_log_error "****************************************************** "
-                exit 22
+                safe_exit 22
             fi
         fi
 
@@ -2478,7 +2518,7 @@ then
             wsrep_log_error "Line $LINENO"
             cat_file_to_stderr "${DATA}/innobackup.decompress.log" "ERR" "innobackup.decompress.log"
             wsrep_log_error "****************************************************** "
-            exit 22
+            safe_exit 22
         fi
 
         wsrep_log_info "Preparing the backup at ${DATA}"
@@ -2496,7 +2536,7 @@ then
             wsrep_log_error "Line $LINENO"
             cat_file_to_stderr "${DATA}/innobackup.prepare.log" "ERR" "innobackup.prepare.log"
             wsrep_log_error "****************************************************** "
-            exit 22
+            safe_exit 22
         fi
 
         XB_GTID_INFO_FILE_PATH="${TDATA}/${XB_GTID_INFO_FILE}"
@@ -2521,7 +2561,7 @@ then
             wsrep_log_error "Line $LINENO"
             cat_file_to_stderr "${DATA}/innobackup.move.log" "ERR" "innobackup.move.log"
             wsrep_log_error "****************************************************** "
-            exit 22
+            safe_exit 22
         fi
 
         wsrep_log_debug "Move successful, removing ${DATA}"
@@ -2529,6 +2569,8 @@ then
         DATA=${TDATA}
         transfer_type="sst"
     else
+        SAFE_EXIT_CODE_OVERRIDE=
+
         wsrep_log_info "${IST_FILE} received from donor: Running IST"
         transfer_type="ist"
     fi
@@ -2538,7 +2580,7 @@ then
         wsrep_log_error "SST magic file ${XB_GTID_INFO_FILE_PATH} not found/readable"
         wsrep_log_error "Line $LINENO"
         wsrep_log_error "****************************************************** "
-        exit 2
+        safe_exit 2
     fi
 
     #-----------------------------------------------------------------------
@@ -2556,7 +2598,7 @@ then
     set -e
     if [[ $errcode -ne 0 ]]; then
         wsrep_log_info "...........post-processing failed.  Exiting"
-        exit $errcode
+        safe_exit $errcode
     else
         wsrep_log_info "...........post-processing done"
     fi
@@ -2568,4 +2610,4 @@ then
     fi
 fi
 
-exit 0
+safe_exit 0
