@@ -1,4 +1,4 @@
-/* Copyright (c) 2004, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2004, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -237,7 +237,7 @@ void make_valid_column_names(LEX *lex) {
     for (Item *item : sl->visible_fields()) {
       ++column_no;
       if (!item->item_name.is_autogenerated() ||
-          !check_column_name(item->item_name.ptr()))
+          !check_column_name(item->item_name))
         continue;
       name_len = snprintf(buff, NAME_LEN, SYNTHETIC_FIELD_NAME "%u", column_no);
       item->orig_name = item->item_name;
