@@ -11,22 +11,30 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+   USA */
 
-#include <my_global.h>
-#include <violite.h>
-#include <sql_class.h>
-#include <sql_connect.h>
-#include <sql_audit.h>
-#include <debug_sync.h>
-#include <threadpool.h>
-#include <probes_mysql.h>
-#include <my_thread_local.h>
-#include <mysql/psi/mysql_idle.h>
-#include <conn_handler/channel_info.h>
-#include <conn_handler/connection_handler_manager.h>
-#include <mysqld_thd_manager.h>
-#include <mysql/thread_pool_priv.h>
+#ifdef WITH_WSREP
+#include "mysql/components/services/log_builtins.h"
+#include "wsrep_trans_observer.h"
+#endif /* WITH_WSREP */
+
+#include "my_thread_local.h"
+#include "mysql/psi/mysql_idle.h"
+#include "mysql/psi/mysql_socket.h"
+#include "mysql/thread_pool_priv.h"
+#include "sql/conn_handler/channel_info.h"
+#include "sql/conn_handler/connection_handler_manager.h"
+#include "sql/debug_sync.h"
+#include "sql/mysqld.h"
+#include "sql/mysqld_thd_manager.h"
+#include "sql/sql_audit.h"
+#include "sql/sql_class.h"
+#include "sql/sql_connect.h"
+#include "sql/protocol_classic.h"
+#include "sql/sql_parse.h"
+#include "sql/threadpool.h"
+#include "violite.h"
 
 /* Threadpool parameters */
 
@@ -40,15 +48,12 @@ uint threadpool_oversubscribe;
 /* Stats */
 TP_STATISTICS tp_stats;
 
-
-extern bool do_command(THD*);
-
 /*
   Worker threads contexts, and THD contexts.
   =========================================
-  
-  Both worker threads and connections have their sets of thread local variables 
-  At the moment it is mysys_var (this has specific data for dbug, my_error and 
+
+  Both worker threads and connections have their sets of thread local variables
+  At the moment it is mysys_var (this has specific data for dbug, my_error and
   similar goodies), and PSI per-client structure.
 
   Whenever query is executed following needs to be done:
@@ -59,68 +64,67 @@ extern bool do_command(THD*);
   3. Process query
   4. Restore worker thread context.
 
-  Connection login and termination follows similar schema w.r.t saving and 
-  restoring contexts. 
+  Connection login and termination follows similar schema w.r.t saving and
+  restoring contexts.
 
-  For both worker thread, and for the connection, mysys variables are created 
+  For both worker thread, and for the connection, mysys variables are created
   using my_thread_init() and freed with my_thread_end().
 
 */
-class Worker_thread_context
-{
+class Worker_thread_context {
 #ifdef HAVE_PSI_THREAD_INTERFACE
-  PSI_thread * const psi_thread;
+  PSI_thread *const psi_thread;
 #endif
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   const my_thread_id thread_id;
 #endif
-
-public:
-  Worker_thread_context()
-#if defined(HAVE_PSI_THREAD_INTERFACE) || !defined(DBUG_OFF)
-    :
-#endif
+ public:
+  Worker_thread_context() noexcept
+      :
 #ifdef HAVE_PSI_THREAD_INTERFACE
-    psi_thread(PSI_THREAD_CALL(get_thread)())
-#ifndef DBUG_OFF
-    ,
+        psi_thread(PSI_THREAD_CALL(get_thread)())
 #endif
-#endif
-#ifndef DBUG_OFF
-    thread_id(my_thread_var_id())
+#ifndef NDEBUG
+        ,
+        thread_id(my_thread_var_id())
 #endif
   {
   }
 
-  ~Worker_thread_context()
-  {
+  ~Worker_thread_context() noexcept {
 #ifdef HAVE_PSI_THREAD_INTERFACE
     PSI_THREAD_CALL(set_thread)(psi_thread);
 #endif
-#ifndef DBUG_OFF
+#ifndef NDEBUG
     set_my_thread_var_id(thread_id);
 #endif
-    pthread_setspecific(THR_THD, 0);
-    pthread_setspecific(THR_MALLOC, 0);
+    THR_MALLOC = nullptr;
   }
 };
 
-
 /*
   Attach/associate the connection with the OS thread,
 */
-static bool thread_attach(THD* thd)
-{
-#ifndef DBUG_OFF
+static bool thread_attach(THD *thd) {
+#ifdef WITH_WSREP
+  /* Wait until possible background rollback has finished before
+     attaching the thd. */
+  wsrep_wait_rollback_complete_and_acquire_ownership(thd);
+
+  DEBUG_SYNC(thd, "thread_attach_after_acquire_ownership");
+
+#endif /* WITH_WSREP */
+#ifndef NDEBUG
   set_my_thread_var_id(thd->thread_id());
 #endif
-  thd->thread_stack=(char*)&thd;
+  thd->thread_stack = (char *)&thd;
   thd->store_globals();
 #ifdef HAVE_PSI_THREAD_INTERFACE
   PSI_THREAD_CALL(set_thread)(thd->get_psi());
+  mysql_thread_set_psi_THD(thd);
 #endif
-  mysql_socket_set_thread_owner(thd->get_protocol_classic()->get_vio()
-                                ->mysql_socket);
+  mysql_socket_set_thread_owner(
+      thd->get_protocol_classic()->get_vio()->mysql_socket);
   return 0;
 }
 
@@ -128,50 +132,40 @@ static bool thread_attach(THD* thd)
 extern PSI_statement_info stmt_info_new_packet;
 #endif
 
-static void threadpool_net_before_header_psi_noop(struct st_net * /* net */,
+static void threadpool_net_before_header_psi_noop(NET * /* net */,
                                                   void * /* user_data */,
-                                                  size_t /* count */)
-{ }
+                                                  size_t /* count */) {}
 
-static void threadpool_init_net_server_extension(THD *thd)
-{
+static void threadpool_init_net_server_extension(THD *thd) {
 #ifdef HAVE_PSI_INTERFACE
   // socket_connection.cc:init_net_server_extension should have been called
   // already for us. We only need to overwrite the "before" callback
-  DBUG_ASSERT(thd->m_net_server_extension.m_user_data == thd);
-  thd->m_net_server_extension.m_before_header=
-    threadpool_net_before_header_psi_noop;
+  assert(thd->m_net_server_extension.m_user_data == thd);
+  thd->m_net_server_extension.m_before_header =
+      threadpool_net_before_header_psi_noop;
 #else
-  DBUG_ASSERT(thd->get_protocol_classic()->get_net()->extension == NULL);
+  assert(thd->get_protocol_classic()->get_net()->extension == NULL);
 #endif
 }
 
-int threadpool_add_connection(THD* thd)
-{
-  int retval=1;
+int threadpool_add_connection(THD *thd) {
+  int retval = 1;
   Worker_thread_context worker_context;
 
   my_thread_init();
 
   /* Create new PSI thread for use with the THD. */
 #ifdef HAVE_PSI_THREAD_INTERFACE
-  thd->set_psi(PSI_THREAD_CALL(new_thread)(key_thread_one_connection, thd,
+  thd->set_psi(PSI_THREAD_CALL(new_thread)(key_thread_one_connection, 0, thd,
                                            thd->thread_id()));
 #endif
 
-
   /* Login. */
   thread_attach(thd);
-  thd->start_utime= thd->thr_create_utime= my_micro_time();
+  thd->start_utime = my_micro_time();
+  thd->store_globals();
 
-  if (thd->store_globals())
-  {
-    close_connection(thd, ER_OUT_OF_RESOURCES);
-    goto end;
-  }
-
-  if (thd_prepare_connection(thd, false))
-  {
+  if (thd_prepare_connection(thd)) {
     goto end;
   }
 
@@ -179,30 +173,26 @@ int threadpool_add_connection(THD* thd)
     Check if THD is ok, as prepare_new_connection_state()
     can fail, for example if init command failed.
   */
-  if (thd_connection_alive(thd))
-  {
-    retval= 0;
+  if (thd_connection_alive(thd)) {
+    retval = 0;
     thd_set_net_read_write(thd, 1);
-    thd->skip_wait_timeout= true;
+    thd->skip_wait_timeout = true;
     MYSQL_SOCKET_SET_STATE(thd->get_protocol_classic()->get_vio()->mysql_socket,
                            PSI_SOCKET_STATE_IDLE);
-    thd->m_server_idle= true;
+    thd->m_server_idle = true;
     threadpool_init_net_server_extension(thd);
   }
 
 end:
-  if (retval)
-  {
-    Connection_handler_manager *handler_manager=
-      Connection_handler_manager::get_instance();
+  if (retval) {
+    Connection_handler_manager *handler_manager =
+        Connection_handler_manager::get_instance();
     handler_manager->inc_aborted_connects();
   }
   return retval;
 }
 
-
-void threadpool_remove_connection(THD *thd)
-{
+void threadpool_remove_connection(THD *thd) {
   Worker_thread_context worker_context;
 
   thread_attach(thd);
@@ -215,60 +205,54 @@ void threadpool_remove_connection(THD *thd)
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
   PSI_THREAD_CALL(delete_thread)(thd->get_psi());
+  mysql_thread_set_psi_THD(nullptr);
 #endif
 
   Global_THD_manager::get_instance()->remove_thd(thd);
-  Connection_handler_manager::dec_connection_count(false);
+  Connection_handler_manager::dec_connection_count();
   delete thd;
 }
 
 /**
  Process a single client request or a single batch.
 */
-int threadpool_process_request(THD *thd)
-{
-  int retval= 0;
+int threadpool_process_request(THD *thd) {
+  int retval = 0;
   Worker_thread_context worker_context;
 
   thread_attach(thd);
 
-  if (thd->killed == THD::KILL_CONNECTION)
-  {
-    /* 
-      killed flag was set by timeout handler 
+  if (thd->killed == THD::KILL_CONNECTION) {
+    /*
+      killed flag was set by timeout handler
       or KILL command. Return error.
     */
-    retval= 1;
+    retval = 1;
     goto end;
   }
 
-
   /*
     In the loop below, the flow is essentially the copy of thead-per-connections
     logic, see do_handle_one_connection() in sql_connect.c
 
-    The goal is to execute a single query, thus the loop is normally executed 
-    only once. However for SSL connections, it can be executed multiple times 
-    (SSL can preread and cache incoming data, and vio->has_data() checks if it 
+    The goal is to execute a single query, thus the loop is normally executed
+    only once. However for SSL connections, it can be executed multiple times
+    (SSL can preread and cache incoming data, and vio->has_data() checks if it
     was the case).
   */
-  for(;;)
-  {
+  for (;;) {
     Vio *vio;
     thd_set_net_read_write(thd, 0);
 
-    if ((retval= do_command(thd)) != 0)
-      goto end;
+    if ((retval = do_command(thd)) != 0) goto end;
 
-    if (!thd_connection_alive(thd))
-    {
-      retval= 1;
+    if (!thd_connection_alive(thd)) {
+      retval = 1;
       goto end;
     }
 
-    vio= thd->get_protocol_classic()->get_vio();
-    if (!vio->has_data(vio))
-    { 
+    vio = thd->get_protocol_classic()->get_vio();
+    if (!vio->has_data(vio)) {
       /* More info on this debug sync is in sql_parse.cc*/
       DEBUG_SYNC(thd, "before_do_command_net_read");
       thd_set_net_read_write(thd, 1);
@@ -277,21 +261,21 @@ int threadpool_process_request(THD *thd)
     if (!thd->m_server_idle) {
       MYSQL_SOCKET_SET_STATE(vio->mysql_socket, PSI_SOCKET_STATE_IDLE);
       MYSQL_START_IDLE_WAIT(thd->m_idle_psi, &thd->m_idle_state);
-      thd->m_server_idle= true;
+      thd->m_server_idle = true;
     }
   }
 
 end:
+  mysql_thread_set_psi_THD(nullptr);
   if (!retval && !thd->m_server_idle) {
-    MYSQL_SOCKET_SET_STATE(thd->get_protocol_classic()->get_vio()
-                           ->mysql_socket, PSI_SOCKET_STATE_IDLE);
+    MYSQL_SOCKET_SET_STATE(thd->get_protocol_classic()->get_vio()->mysql_socket,
+                           PSI_SOCKET_STATE_IDLE);
     MYSQL_START_IDLE_WAIT(thd->m_idle_psi, &thd->m_idle_state);
-    thd->m_server_idle= true;
+    thd->m_server_idle = true;
   }
+
   return retval;
 }
 
-THD_event_functions tp_event_functions=
-{
-  tp_wait_begin, tp_wait_end, tp_post_kill_notification
-};
+THD_event_functions tp_event_functions = {tp_wait_begin, tp_wait_end,
+                                          tp_post_kill_notification};
