@@ -829,9 +829,7 @@ static inline Rec_instant_state rec_convert_dtuple_to_rec_comp(
         }
         break;
       case REC_STATUS_NODE_PTR:
-        ut_ad(n_fields ==
-              static_cast<ulint>(
-                  dict_index_get_n_unique_in_tree_nonleaf(index) + 1));
+        ut_ad(n_fields == dict_index_get_n_unique_in_tree_nonleaf(index) + 1U);
         n_node_ptr_field = n_fields - 1;
         n_null = index->get_nullable_before_instant_add_drop();
         break;
